import {
	BindService,
	BindServiceImpl,
} from './bind.service';
import { CommandHandler } from '../../packages/cli-wrapper';

export interface BindController {
	bindBook: CommandHandler;
}

export class BindControllerImpl implements BindController {
	constructor(
		private readonly bindService: BindService = new BindServiceImpl(),
	) {
		// noop
	}

	readonly bindBook: CommandHandler = async (params) => {
		if (!params.interactive) {
			const checkArgs = params.args as Record<string, unknown>;
			const checkFormat = checkArgs.format ?? checkArgs.f;
			if (typeof checkFormat === 'undefined') {
				params.logger.error('Missing required argument: format');
				return -1;
			}
			const checkOutputPath = checkArgs.outputPath ?? checkArgs.o;
			if (typeof checkOutputPath === 'undefined') {
				params.logger.error('Missing required argument: outputPath');
				return -1;
			}
		}

		const { inputPath: inputPathRaw, f, format = f, o, outputPath = o } = params.args;
		const inputPath = inputPathRaw ?? process.cwd();

		try {
			const response = await this.bindService.bindBook({
				input: {
					path: inputPath,
				},
				output: {
					format,
					path: outputPath
				},
			});
			params.logger.info(response);
		} catch (errorRaw) {
			console.error(errorRaw);
			const error = errorRaw as Error;
			params.logger.error(error.message);
			return -2;
		}
		return 0;
	}
}
