import {
	bindBook,
	BindFunctionOptions,
} from '@modal-sh/patchouli-core';
import * as v from 'valibot';
import { writeFile } from 'fs/promises';
import { resolve } from 'path';

const outputOptionSchema = v.object({
	path: v.string()
});

interface BindOptions extends BindFunctionOptions {
	output: BindFunctionOptions['output'] & v.Output<typeof outputOptionSchema>;
}

export interface BindService {
	bindBook(options: BindOptions): Promise<string>;
}

export class BindServiceImpl implements BindService {
	async bindBook(options: BindOptions) {
		const parsedOptions = await v.parseAsync(outputOptionSchema, options.output);
		const effectiveOptions = {
			...options,
			output: {
				...options.output,
				...parsedOptions,
			},
		};
		const buffer = await bindBook(effectiveOptions);
		const outputPath = effectiveOptions.output.path;
		await writeFile(outputPath, buffer);
		const absolutePath = resolve(outputPath);
		return `File successfully written to ${absolutePath}.`;
	}
}
