import {readdir, stat,  readFile} from 'fs/promises';
import {resolve} from 'path';
import * as v from 'valibot';
import {BINDING_FILENAME, bindingFileBaseSchema, BOOK_FILENAME, bookFileSchema, Input} from '../../common';

export const name = 'path' as const;

const inputSchema = v.object({
	path: v.string(),
});

interface PathInput extends Input, v.Output<typeof inputSchema> {}

const readPath = async (path: string, rootPath = path, readFiles = []) => {
	const files = await readdir(path);

	// TODO get the buffers on the tree
	//console.log(files);
};

export class InvalidInputPathError extends Error {}

type Book = v.Output<typeof bookFileSchema>;

type Binding = v.Output<typeof bindingFileBaseSchema>;

const getBookFile = async (bookFilePath: string): Promise<Book | undefined> => {
	const bookFileString = await readFile(bookFilePath, 'utf-8');
	const bookFileRaw = JSON.parse(bookFileString);
	return await v.parseAsync(bookFileSchema, bookFileRaw);
};

const getBindingFile = async (bindingFilePath: string, defaultBinding: Binding) => {
	const bindingFileString = await readFile(bindingFilePath, 'utf-8');
	const bindingFileRaw = JSON.parse(bindingFileString);

	return {
		...defaultBinding,
		...bindingFileRaw,
	};
};

export const compileFromInput = async <T extends PathInput = PathInput>(input: T) => {
	const files = await readdir(input.path);
	if (!files.includes(BOOK_FILENAME)) {
		throw new InvalidInputPathError(`Path does not contain a "${BOOK_FILENAME}" file.`);
	}

	const bookFilePath = resolve(input.path, BOOK_FILENAME);
	const bookFile = await getBookFile(bookFilePath);

	const defaultBinding = {
		generatorType: 'static',
		// TODO should make the dist directory related to book file when getting contents
		generatorDistDirectory: resolve(input.path, 'dist'),
	};

	const bindingFilePath = resolve(input.path, BINDING_FILENAME);
	const bindingFile = files.includes(BINDING_FILENAME)
		? await getBindingFile(bindingFilePath, defaultBinding)
		: defaultBinding;

	return [

	];
};
