import * as v from 'valibot';
import assert from 'assert';
import * as PdfFormat from './formats/pdf';
import * as EpubFormat from './formats/epub';
import * as PathCompiler from './compilers/path';
import {inputSchema} from './common';

const AVAILABLE_COMPILERS = [
	PathCompiler,
];

const AVAILABLE_FORMATS = [
	PdfFormat,
	EpubFormat
];

const optionsSchema = v.object(
	{
		input: v.merge(
			[
				inputSchema,
				v.object({
					sourceType: v.picklist(AVAILABLE_COMPILERS.map((f) => f.name)),
				})
			],
			v.unknown(),
		),
		output: v.object(
			{
				format: v.picklist(AVAILABLE_FORMATS.map((f) => f.name)),
			},
			v.unknown(),
		),
	},
	v.never(),
)

export interface BindFunctionOptions extends v.Output<typeof optionsSchema> {}

export interface BindFunction {
	<T extends BindFunctionOptions = BindFunctionOptions>(options: T): Promise<Buffer>;
}

export const bindBook: BindFunction = async (options: BindFunctionOptions): Promise<Buffer> => {
	const { input, output, } = await v.parseAsync(optionsSchema, options);
	const selectedCompiler = AVAILABLE_COMPILERS.find((c) => c.name === input.sourceType);

	const selectedFormat = AVAILABLE_FORMATS.find((f) => f.name === output.format);
	assert(typeof selectedFormat !== 'undefined');

	return selectedFormat.bindBook(input);
};
